/*********************************************************
 *
 * word completation
 *
 * Copyright (c) 2001 Christian Ensel
 *
 *********************************************************/

/* PVMerlin-dependent part */
#define ALLOW_KW_ADDONLOAD  (PARAM(PARAM_KEYWORDS) != P_KW_OFF && PARAM(PARAM_KEYWORDS) != P_KW_ON_NOLOAD )
#define ALLOW_KW_ADDONWRITE (PARAM(PARAM_KEYWORDS) != P_KW_OFF)
#define ALLOW_KW_OFFER      (PARAM(PARAM_KEYWORDS) != P_KW_OFF)  /* also disallows add_on_write */
/* plus debug stuff */
/*end PVMERLIN dependent part */


#include <stdrom.h>
#include "define.h"
#include "libc.h"

#include "util.h"
#include "params.h"
#include "keywords.h"

#define CPNOWHERE -2

static char inpword[MAXKEYWORDLEN+1];
static int i_inpword;
static char keywordbuf[MAXKEYWORDS][MAXKEYWORDLEN+1];
static bool windowvisible;
static int nkw_inbuffer;
static int lastcp;    

    
static TCHTBL kw_tchtbl[2];

bool isInternationalAlphaNum(char c)
{
    if ( isalnum(c) )
        return TRUE;
    if ( c>=0x80 && c<=0x8e || 
         c>=0x90 && c<=0x9e || 
         c>=0xa0 && c<=0xab ||
         c==0xad || 
         c>=0xb0 && c<=0xb7 || 
         c==0xb9 || 
         c==0xba )
        return TRUE;

#ifdef POLISH
    if ( c==0x0b || c==0x7f || c==0xd2 || c==0xe8 || 
         c>=0xea && c<=0xef )
        return TRUE;
#endif

    return FALSE;
}

/* call once at prog-start */
void initKeyword()
{
    kw_tchtbl[0].x1 = 30;
    kw_tchtbl[0].y1 = 30; 
    kw_tchtbl[0].x2 = 140;
    kw_tchtbl[0].y2 = 45;
    kw_tchtbl[0].act = ACT_MAKE;
    kw_tchtbl[0].obj = OBJ_KEYWORD;
    kw_tchtbl[0].ext = 0; 
    kw_tchtbl[1] = TCHTBL_END;
     
    resetKeywordprefix();
    /*ce-new2 delline LibKeyWordInit(); */
    windowvisible=FALSE;
}


void addKeywords(char *txt)
{
    char buf[MAXKEYWORDLEN+1];
    int l=0;

    if ( ! ALLOW_KW_ADDONLOAD 
#ifdef TEST    
         || inSecretMode() /*ceV3.2*/ 
#endif         
       ) 
        return;
            
    while (*txt!=0) {
        buf[l] = *txt++;
        if ( !isInternationalAlphaNum(buf[l]) || l>=MAXKEYWORDLEN ) {
            if ( l >= MINKEYWORDLEN ) {
                buf[l] = 0;
                LibKeyWordSet(buf);
#ifdef ALLOWDEBUGCODE
                if ( PARAM(PARAM_DEBUG) == P_DBG_FULL || PARAM(PARAM_DEBUG) == P_DBG_KEYWORDS ) {
                    LibPutMsgDlg2("kw:%s", buf );
                }
#endif
            }
            l = 0;
        }
        else
            l++;
    }
}


bool keywordsAvailable()
{
    return (windowvisible);
}


/* show window with keywords, add window touchtable */
/* (x,y) is 'close' to window, but always left uncovered by window 
   for small ys window is below coordinates, for higher ones it is above */ 

void openKeywordWindow(int x, int y)
{
    int i;
    int xsize=0;
    int ysize = 3 + nkw_inbuffer*9;
    if (!windowvisible)
        return; 
    for ( i=0; i<nkw_inbuffer; i++)
        xsize = max(xsize, LibGetProStrSize(IB_PFONT1, keywordbuf[i]));
    xsize += 5;
    if (xsize>160)
        xsize = 160;
    kw_tchtbl[0].x1 = max ( min(160-xsize, x-i_inpword*5-2 ), 3); /* 5 is average character-width */
    if (kw_tchtbl[0].x1+xsize>160)
        kw_tchtbl[0].x1=0;
    kw_tchtbl[0].y1 = y<72-ysize ? y+9 : y-ysize;  /* if window fits on upper half, make it below y, otherwise above */
    if (kw_tchtbl[0].y1+ysize>158)
        kw_tchtbl[0].y1 = min(y-ysize, 157-ysize);
    kw_tchtbl[0].x2 = kw_tchtbl[0].x1 + xsize -1;
    kw_tchtbl[0].y2 = kw_tchtbl[0].y1 + ysize -1;
    LibOpenWindow( kw_tchtbl[0].x1, kw_tchtbl[0].y1, xsize, ysize );
    LibBox( kw_tchtbl[0].x1, kw_tchtbl[0].y1, xsize, ysize, 1);
    for ( i=0; i<nkw_inbuffer; i++)
        LibPutProStr(IB_PFONT1, kw_tchtbl[0].x1 + 3, kw_tchtbl[0].y1+2+i*9, keywordbuf[i], xsize-4 );
    LibTchStackPush(kw_tchtbl);
    LibPutDispBox(kw_tchtbl[0].x1, kw_tchtbl[0].y1, xsize, ysize );
}


void closeKeywordWindow()
{
    windowvisible = FALSE;
    LibCloseWindow();
    LibTchStackPop();
}

void putDispAtKeywordWindow()
{
    LibPutDispBox(kw_tchtbl[0].x1, kw_tchtbl[0].y1, kw_tchtbl[0].x2-kw_tchtbl[0].x1+1, kw_tchtbl[0].y2-kw_tchtbl[0].y1+1 );
}



char* getKeywordRemainder(TCHSTS* ptsts)
{
    int sel;
    if (ptsts->obj != OBJ_KEYWORD || nkw_inbuffer<=0 )
        return NULL;
    
    /* find selected line */
    sel =  (ptsts->y - kw_tchtbl[0].y1 - 2) / 9;
    if (sel<0 || sel>=nkw_inbuffer)
        sel = nkw_inbuffer-1;
    return (keywordbuf[sel] + strlen(inpword));
}


char* getKeywordRemainderAt(int sel)
{
    if (nkw_inbuffer<=0 )
        return NULL;
    if (sel<0 || sel>=nkw_inbuffer)
        sel = nkw_inbuffer-1;
    return (keywordbuf[sel] + strlen(inpword));
}


void resetKeywordprefix()
{
    inpword[0]=0;
    i_inpword = 0;
    lastcp = CPNOWHERE;
    nkw_inbuffer=0;
}

void rereadKeywordPrefix(char *txt, int cp)
{
    char *w = txt + cp -1;
    if (!isInternationalAlphaNum(*w)) /* jump over appended non-alpha-num-char */
        w--;
    while ( w>=txt && isInternationalAlphaNum(*w) )
        w--;  /* find start of word */
    w++;
    resetKeywordprefix();
    while ( w < txt + cp-1 && *w!=0 && i_inpword<MAXKEYWORDLEN) {
        inpword[i_inpword++] = *w++;
    }               
    inpword[i_inpword]=0;
}


void delLastCharFromKeywordPrefix()
{
    if (i_inpword >0)
        i_inpword--;
    inpword[i_inpword] = 0;
    lastcp--;
}


/* internal fct. (see below) */
static void myCheckKeywordPrefix(char *txt, int cp, int doCheck)
{
    char ch; /*, alt;*/
    int i;
    
    if ( !ALLOW_KW_OFFER 
#ifdef TEST    
         || inSecretMode() /*ceV3.2*/ 
#endif         
       )
        return;
        
    if ( cp==lastcp ) /* nothing changed */
        return;
        
    if ( cp==0 || (lastcp!=CPNOWHERE && cp != lastcp+1) ) { /* cursor jumped around */
#ifdef ALLOWDEBUGCODE
        if ( PARAM(PARAM_DEBUG) == P_DBG_FULL || PARAM(PARAM_DEBUG) == P_DBG_KEYWORDS ) {
            LibPutMsgDlg3("cursor jump" );
        }
#endif
        resetKeywordprefix();
        lastcp=cp;
        return;
    }

    lastcp=cp;
    ch = txt[cp-1];
    
    if ( isInternationalAlphaNum(ch) ) {
        /* add char */ 
        inpword[i_inpword] = ch;
        i_inpword++;
        inpword[i_inpword] = 0;
#ifdef ALLOWDEBUGCODE
        if ( PARAM(PARAM_DEBUG) == P_DBG_FULL || PARAM(PARAM_DEBUG) == P_DBG_KEYWORDS ) 
            LibPutMsgDlg2("add char %c",ch);
#endif
        /* check words */
        if ( doCheck && i_inpword>1 && LibKeyWordFSrch(inpword,keywordbuf[0])==TRUE ) {
#ifdef ALLOWDEBUGCODE
            if ( PARAM(PARAM_DEBUG) == P_DBG_FULL || PARAM(PARAM_DEBUG) == P_DBG_KEYWORDS ) 
                LibPutMsgDlg2("match");
#endif
            nkw_inbuffer = 1;
            for ( i=1; i<MAXKEYWORDS && LibKeyWordNSrch(inpword,keywordbuf[i]); i++ )
                nkw_inbuffer++;
            /*ce-new2 del for (; i<MAXKEYWORDS; i++)
                keywordbuf[i][0]=0;*/
            windowvisible = TRUE;
        } 
        /*ce-new2 offer alternative characters for last input * 
        does not work well
        if ( nkw_inbuffer < MAXKEYWORDS && (alt=getInputAlternative()) != 0 ) { 
            char *alternative = "\x0cf ";
            nkw_inbuffer = 1;
            windowvisible = TRUE;
            alternative[1] = alt;
            strcpy(keywordbuf[0],alternative);
            clearInputAlternative();
        }
        *ce-new2-end*/ 
    }
    else { /* seperating character */
        if ( i_inpword>=MINKEYWORDLEN && ALLOW_KW_ADDONWRITE) {
            /* add new keyword */
            rereadKeywordPrefix(txt,cp);
            /*LibPutMsgDlg3("add word: %s", inpword ); */
#ifdef ALLOWDEBUGCODE
            if ( PARAM(PARAM_DEBUG) == P_DBG_FULL || PARAM(PARAM_DEBUG) == P_DBG_KEYWORDS ) {
                LibPutMsgDlg3("kw:%s", inpword );
            }
#endif
            if ( i_inpword>0 )
                LibKeyWordSet(inpword);
        }
        resetKeywordprefix();
    }
}

/* - adds chars before cursor position cp in textbuffer txt to current
     keyword-prefix (=new input compared with stored keywords).
   - checks if current keywords match prefix
   - adds new words to keyword-buffer, if a word is completed
   - function able to cope with cursor replacements  */
void checkKeywordPrefix(char *txt, int cp) {
    myCheckKeywordPrefix(txt, cp, TRUE);
}

/* same, but dont check keywords, just update keyword prefix */
void updateKeywordPrefix(char *txt, int cp) {
    myCheckKeywordPrefix(txt, cp, FALSE);
}

