/*********************************************************
 *
 * Loading and saving memo items.
 *
 * Copyright (c) 2001,2002 Marcin Woli\'nski & Christian Ensel
 *
 * Marcin Woli\'nski 2002/09/10
 * Christian Ensel   2002/08/27
 * Marcin Woli\'nski 2002/02/17
 * Christian Ensel   2001/09/27
 * Marcin Woli\'nski 2001/08/12
 *
 *********************************************************/

#include <stdrom.h>
#include "define.h"
#include "libc.h"

#include "util.h"   
#include <loadsave.h>
#include "params.h" 

#define NUM_CATEGORIES 5
#define LOADDIALOG_CATEGORY SYS_MEMO_CATEGORY
#define CURRENT_FILEREAD_MAIN_ENTR(SUBCAT) 3 << 4 /* Memo  */
#define CURRENT_FILEWRITE_MAIN_ENTR(SUBCAT) 3 << 4 /* Memo  */
#define CONTACTBUSINESSSORTMODE(SUBCAT) 0

#ifdef TEST
  #define MAXPREVIEWLINES 18
  #define MAXPREVIEWWIDTH 25
  #define MAXPREVIEW MAXPREVIEWLINES*(MAXPREVIEWWIDTH+2)
  static char previewbuf[MAXPREVIEW+3];
  static char *previewline[MAXPREVIEWLINES+1];
#endif

/*static char* getNextPrevPageContents(char *title, t_searchdir d, word *p_fp, int cat);*/
static char* getNextPrevPageContentsAndCat(char *title, t_searchdir d, word *p_fp, int *p_cat);

extern int applyParam(int p); /* in PV.c */
extern int drawMainIcon(); /*ce implemented in PV.c */
extern int drawProgramName(int x, int isLong); /*ceV3.1 implemented in PV.c */

int istitleendchar(char c) {
    return (c=='-' || c==':' || c=='(' || c=='[' || c=='\r' || c==0 || c==0xfe );
}    

/* Dramatis personae:

   FLCOMBO is the combobox displaying categories */
#define FLCOMBO 0
#define OBJ_FLCOMBO 0x8100
/* the 'New' button: */
#define FLNEW 1
#define OBJ_FLNEW 0x8101
/* the 'Tools' button: */
#define FLTOOLS 2
#define OBJ_FLTOOLS 0x8102

#ifdef TEST  /*ce preview button*/
  #define FLPREVIEWBUTTONX 149
  #define FLPREVIEWBUTTONY 15
  #define FLPREVIEWBUTTON 3
  #define OBJ_FLPREVIEWBUTTON 0x8110
#endif  

#define OBJ_FLLIST 0x8107  /* the listbox: */
#define OBJ_FLSCB 0x8108   /* and its scrollbar */


static TCHTBL far TchFL[] = {
    {0,13, 136,24, ACT_ICON, OBJ_FLCOMBO, 0},
    {98,0, 128,11, ACT_ICON, OBJ_FLNEW, 0},
    {130,0, 159,11, ACT_ICON, OBJ_FLTOOLS, 0},
#ifdef TEST
    {FLPREVIEWBUTTONX,FLPREVIEWBUTTONY-1, FLPREVIEWBUTTONX+10,FLPREVIEWBUTTONY+8, ACT_ICON, OBJ_FLPREVIEWBUTTON, 0},
#endif
    {0,27, 148,157, ACT_MAKE, OBJ_FLLIST, 0},
    {147,27, 159,157, ACT_SCR_BAR, OBJ_FLSCB, 0},
    {0,0, 0,0, 0,OBJ_END,0 }
};

static T_ICON far FLIcons[] = {
    {&TchFL[FLCOMBO],NULL,NULL,0x03},
    {&TchFL[FLNEW],NULL,NULL,0x02},
    {&TchFL[FLTOOLS],NULL,NULL,0x02},
#ifdef TEST
    {&TchFL[FLPREVIEWBUTTON],NULL,NULL,0x03},
#endif
};

static T_SCR_POS FLScb;
static int sel;          /* selected item */

/*MW:
   Since the number of memos in any category is essentialy unlimited
   (well, limited by the range of short integers) and we have no
   dynamic memory allocation, we'll keep only those items in memory,
   that are currently displayed on the screen.
   For each memo item we need to remember the title (for displaying it) and the
   file pointer (to be able to load that item easily).
*/

/* number of items that fit on the display */
#define DISPITEMS 13 
/* max length of a memo title */
#define MAXTITLE 45
static char titles[DISPITEMS][MAXTITLE+1];  /* list of memo titles */
static word fps[DISPITEMS]; /* file pointers for memos listed */
static word cats[DISPITEMS]; /* categories for memos listed () */


/* Copies memo title from 'from' to 'to'.
 * If 'title' is non-null, copy subtitle only.
 */
static void fetchtitle(char *from, char* title, char *to, int contactbusinesssortmode) {
    int charcnt=0;

    if (title) { /* show only subtitle */
	while ( !IS_TITLE_ENDCHAR(*from) ) from++;
	if (*from == '-' || *from == ':' || *from == 0xfe) {  /* '(', '[' remains in subtitle */
	    from++;
	    while ( *from==' ' )  /* skip space */
		from++;
	}
	if (*from==0 || *from=='\r') { /* no title or no subtitle */
	    strcpy(to,"[unnamed page]");
	    to += strlen(to); /* set 'to' at the end again */
	}
    }

    /* copy title */
    while ( *from!=0 
            && *from!='\r'
            && charcnt++<MAXTITLE-1) {
	    *to++=*from++;
    }
    *to='\0';  
}

/*
 * Populates the above tables with file names and pointers of memos of
 * category 'cat'.
 * If 'title' is non-null only shows titles starting with 'title'.
 * If 'shift' is 0 the tables are filled with first DISPITEMS memos.
 * If 'shift' is nonzero current contents of the tables are shifted
 * that much up or down and then additional items are loaded.
 */
static void populatelist(int cat, char *title, const int shift) {
    int i; /* index for loops */
    word fp; /* from that pointer we'll search for more items */
    char *ptext;
    int tmpcat;

    /* prepare display, shift existing data:  */
    if (shift==0) { /* clear the whole list area: */
	fp = 0xffff;
	LibGdsClr(0,27, 148,157);  /* list box */
	LibGdsBox(0,27, 148,157);  /* list box */
    } else if (shift>0) {
	fp = fps[DISPITEMS-1]; /* search will continue from the last
				  displayed item */
	if (shift<DISPITEMS) { /* if any of current items will
				    remain visible: */
			       /*ce bug: was shift<DISPITEMS-1 */
	    LibGrpUp(2,28, 144,10*DISPITEMS-1, 10*shift);
	    for (i=shift; i<DISPITEMS; i++) {
		fps[i-shift]=fps[i];
		cats[i-shift] = cats[i]; /*ce T2*/
		strcpy(titles[i-shift], titles[i]);
	    }
	} else LibGdsClr(1,28, 147,156); /* clear all */
    } else if (shift<0) { /* scroll down */
	fp = fps[0]; /* search will continue from the first displayed
			item */ 
	if (-shift<DISPITEMS) { /*ce bug: was -shift<DISPITEMS-1 */
	    LibGrpDwn(2,28, 144,10*DISPITEMS-1, -10*shift); 
	    for (i=DISPITEMS-1; i>=-shift; i--) {
		fps[i]=fps[i+shift];
		cats[i] = cats[i+shift]; /*ce T2*/
		strcpy(titles[i], titles[i+shift]);
	    }
	} else LibGdsClr(1,28, 147,156);
    }
    
    /* load additional memos: */
    if  (shift == 0) { /* load whole list: */
        tmpcat=cat;
	for ( i=0 ; i<DISPITEMS &&
		  (ptext=getNextPrevPageContentsAndCat(title, SEARCH_NEXT_PAGE, &fp, &tmpcat))
		  ; i++ ) { 
	    fps[i] = fp;
	    cats[i] = tmpcat;
	    tmpcat=cat;
	    fetchtitle(ptext, title, titles[i],CONTACTBUSINESSSORTMODE(cat));
	}
	if (i==DISPITEMS) { /* There may be more items than fit on the screen.
			     We have to get the total number for the scrollbar. */
	    while (tmpcat=cat, getNextPrevPageContentsAndCat(title, SEARCH_NEXT_PAGE, &fp, &tmpcat))
	      i++; 
        }
	FLScb.vol = i; /* initialize the scrollbar */
	FLScb.pos = 0; 
	sel = 0;
    } else if (shift >0) { /* on scroll up */
        tmpcat=cat;
	for ( i=DISPITEMS-shift;
	      i<DISPITEMS && 
		  (ptext=getNextPrevPageContentsAndCat(title, SEARCH_NEXT_PAGE, &fp, &tmpcat))
		  ; i++ ) { 
	    if (i<0) { /* when shift>DISPITEMS we have to scroll through some items */
                tmpcat=cat;
		continue;
	    }
	    fps[i] = fp;
	    cats[i] = tmpcat;
	    tmpcat=cat;
	    fetchtitle(ptext, title, titles[i],CONTACTBUSINESSSORTMODE(cat));
	}
    } else if (shift <0) {
        tmpcat=cat;
	for ( i=-shift-1; i>=0 &&
		  (ptext=getNextPrevPageContentsAndCat(title, SEARCH_PREV_PAGE, &fp, &tmpcat))
		  ; i-- ) { 
	    if (i>DISPITEMS-1) { /* when -shift>DISPITEMS */ 
	        tmpcat=cat;
		continue;
            }
	    fps[i] = fp;
	    cats[i] = tmpcat;
	    tmpcat=cat;
	    fetchtitle(ptext, title, titles[i],CONTACTBUSINESSSORTMODE(cat));
	}
    }

    /* Some items may need drawing: */
    for (i=(shift>0?max(DISPITEMS-shift-1,0):0); /* -1 to redraw one meshed line */
	 i<(shift<0?min(DISPITEMS,-shift):DISPITEMS); i++) { 
	if (FLScb.pos+i < FLScb.vol) 
	    LibPutProStr(IB_PFONT1, 3,29+i*10, titles[i], 145-3);
	if (i==DISPITEMS-1) continue;
	LibGdsLineMesh(3,37+i*10, 145,37+i*10);
    }
    
    sel-=shift;
    if (sel<0) sel=0;
    if (sel>=DISPITEMS) sel=DISPITEMS-1;
}

/* Puts the possibly edited category name for category 'i' into a
 * buffer 'name'.  CAUTION: 'name' must point to a previously
 * allocated buffer of at least 198 bytes (actually only in contacts 
 * for Memo-mode only 16 bytes are sufficient). 
 */
static void getcategoryname(int i, byte *name) {
    if (i==0) 
    	LibReadMessage(31,name); /* Get name for "All" */
    else {
        LibLblAreaRead(name, 4+i); /* Get edited category name... */
        if (*name==0 || *name==0xfe) 
            LibReadMessage(245+i,name); /* or default category name */
    }
    while (*name!=0xfe && *name!=0)
        name++;
    *name=0; /* terminate string with 0 if it was terminated with 0xfe (like in contacts mode) */
}



/* Displays a menu showing memo categories and returns number
 * corresponding to the one selected. 0 is All, 1..NUM_CATEGORIES are real
 * categories.  
 * if title!=NULL displays a title-string and does not allow to select "All"
 */
int selectcategory(int cat, char *title) {
    SLW2_ST mnu;
    byte name[16];
    byte rv[1+NUM_CATEGORIES+1];
    byte sp[(1+NUM_CATEGORIES)*16+2];
#ifdef TEST1
    int first;
#endif

    byte *mp;
    int i;
    mp=sp;
    
    /*  for (i=0;i<=NUM_CATEGORIES;i++) 'All' blocked */
#ifdef TEST1
    first=(title?1:0);
    for (i=first ;i<=NUM_CATEGORIES;i++)
#else
    for (i=1;i<=NUM_CATEGORIES;i++)
#endif
	{
	    getcategoryname(i, name);
	    strncpy(mp,name,15);
	    while (*mp!=0)
	        mp++;
	    *mp++=0xfe;
	    /* rv[i]=i; 'All' blocked */
#ifdef TEST1
	    rv[i-first]=i;
#else
	    rv[i-1]=i;
#endif
	}
    mp--;
    *mp++=0xff; /* end-menu marker */
    *mp=0;

#ifdef TEST1
    rv[(1-first)+NUM_CATEGORIES]=0xff;
    if (cat<0) 
      cat=0;
    else if (cat>NUM_CATEGORIES-first)
      cat=NUM_CATEGORIES-first;
#else
    rv[NUM_CATEGORIES]=0xff;
    if (cat<1) 
      cat=1;
    else if (cat>NUM_CATEGORIES)
      cat=NUM_CATEGORIES;
#endif    
    if ( title ) {
        /* centered menu with title */
        return (SelWindowExt2(-1, -1, title , cat>0 ? cat-1 : 0, sp, rv ));
    }    

    /* category menu without title and fixed width and position */
    mnu.x = 0; mnu.y = 26;
    mnu.xs = 125;
    mnu.ity = 9;

    /* mnu.np = cat; 'All' blocked */
    /*  rv[1+NUM_CATEGORIES]=0xff; 'All' blocked */
#ifdef TEST1
    mnu.np = cat;
#else
    mnu.np = cat-1;
#endif    
    mnu.cmnt = sp;
    mnu.rtv = rv;
    mnu.t_xs = 3;

    return(LibSelWindowExt(&mnu));
}

/* This constant is used as an 'unknown' value for a loop below that should run
   until the return value for the function is known.
*/
#define CONTINUELOOP 3241

static void selBarReverse()
{ 
    LibReverse(2,28+sel*10, 145,9);
}

/* Load new text via user query.
   If title!=NULL it only displays memos with the given title ('goto page' mode).
   return value:
     TRUE: a memo has been loaded
     HALF: <New> button pressed
     FALSE: load cancelled or some error has occured
   iff TRUE: assigns loaded file-pointer to *p_fp and current
   category to *p_cat
*/
int loadtext(char *txtbuf, char* title, word *p_fp, int *p_cat) {
    TCHSTS tsts;
    int cat;
    byte name[16];
    int retval = CONTINUELOOP; /* return value and a flag for the loop below */
    
#ifdef TEST1
    /* displayed cagtegory is either "All" or corresponds with the last loaded file */
    if ( LOADDIALOG_CATEGORY == 0 && !title )
      cat = 0;
    else
#endif
        /* category is initialized to the category of currently edited memo or 1 if no such memo exists */
        cat = (*p_cat < 1) ? 1 : ( (*p_cat>NUM_CATEGORIES) ? NUM_CATEGORIES : *p_cat ); /*ceV3.1 necessary due to bug in pvmerlin3.0 which corrupts category entry in very rare cases*/

    getcategoryname(cat, name);

    /* Draw the dialog in a new window: */
    LibOpenWindow(0,0, 160,160);
    /* Initialize a local touch table: */
    LibTchStackPush(NULL);
    LibTchStackPush(TchHardIcon);
    LibTchStackPush(TchFL);
    LibTchStackPush(TchActionKey); /*actionWheelTch);  /*ceV3.1*/
    LibTchInit();

    FLScb.dsp = DISPITEMS;
    FLScb.x = 150; FLScb.y = 28;
    FLScb.size = 157-28;


    if (!title) {
	LibGdsBox(0,13, 136,24);  /* combo */
	LibPutFarData(125,13, 115);   /* down arrow */
	/*LibPutFarData(0,13, 79); */
	LibPutProStr(IB_PFONT1, 3,15, name, 125-3);
    } else /* cat selection not allowed */
	LibPutProStr(IB_PFONT2, 3,16, title, 150); /* show the title */

    LibGdsBox(0,27, 148,157);  /* list box */

    if (title) /*goto page*/
        LibPutProStr(IB_PFONT3, drawMainIcon()+1, 1 /*ce*/, "Go to page", 95);  /*ceV3.1*/
    else { /*ceV3.1*/
        int x = drawProgramName(drawMainIcon()+1,1)+2; 
        LibPutProFont(IB_PFONT3, '-', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, '-' );
        LibPutMessage(49/*List*/, x+1, 1, IB_PFONT3);
    }
    
    LibPutFarData(98, 0, 131);
    LibPutFarData(130, 0, 131);
    
    /* MW: This is somewhat inconsistent since we're using
       untranslated strings in the main view and translated in the
       load dialog, but let it be.  Christian, I hope you're just
       preparing to internationalize all of the interface ;-) */
    /*ce Right, but e.g. the main menu already says 'Neu' (in German) and
      the main view has a smaller 'Tool' button (not Tools). So I thought
      two internationalized buttons in the load dialog are ok :) */
      
    LibPutMessageCenter2(14/*New*/,   113, 2, IB_PFONT1); /*ce*/
    LibPutMessageCenter2(89/*Tools*/, 145, 2, IB_PFONT1); /*ce*/
    /*ce LibPutProStr(IB_PFONT1, 101,2, "New", 25);
    LibPutProStr(IB_PFONT1, 133,2, "Tools", 25); */
    LibGdsMesh(0,158,159,159); /*ce*/
    LibGdsLine(149,27, 159,27);  /*ce */
    LibGdsLine(149,157, 159,157);  /*ce */
    /*
    LibGdsMesh(0,158,159,159);    /*ce*
    LibGdsLine(149,27, 159,27);   /*ce*
    LibGdsLine(149,157, 159,157); /*ce*
    */
    
    /* LibIconPrint(&FLIcons[FLNEW]);
       LibIconPrint(&FLIcons[FLTOOLS]); */

#ifdef TEST
    LibPutFarData(FLPREVIEWBUTTONX+1, FLPREVIEWBUTTONY, 48 );
#endif

    /* Load items of the selected category: */
    populatelist(cat,title,0);
    if (FLScb.vol >0) /* if there are any items */
  	selBarReverse();  /* highlight one */
    LibScrollPrint(FLScb); /* show the scrollbar */
    if (FLScb.vol<=FLScb.dsp) 
  	LibScrollArrowPrint(FLScb, 0); /* this is strange: the call above renders scrollbar
					  arrows black only if there is more than a screenful of
					  items. */
    /* But we want the down-arrow to be active if there is more than 1 item: */
    LibScrollArrowPrint(FLScb, FLScb.vol>1 ? SCR_UP_MASK : SCR_ALL_MASK);

    LibPutDisp();
    /* LibPutMsgDlg("v: %d p: %d\rs: %d\rarr: %d", 
       FLScb.vol, FLScb.pos, sel,
       FLScb.vol>1 ? SCR_UP_MASK : SCR_ALL_MASK);  */


    /* The event loop: */
    while (retval == CONTINUELOOP) {
	int oldpos, scrlreq;
	oldpos = FLScb.pos; /* remembered to redraw list only when needed */
	LibTchWait(&tsts);
	/*	LibPutMsgDlg("event:\r obj=%x\r act=%x x=%d y=%d ext=%x",
		tsts.obj, tsts.act, tsts.x, tsts.y, tsts.ext); */
	switch (tsts.obj) {
	case OBJ_FLNEW: /* 'New' clicked */
	    if (LibIconClick(&FLIcons[FLNEW], &tsts)) {
		retval = HALF;
	    }
	    continue;
	case OBJ_FLTOOLS: /* 'Tools' clicked */
	    if (LibIconClick(&FLIcons[FLTOOLS], &tsts)) {
		LibFuncPtool();
	    }
	    break;
	case OBJ_FLCOMBO: /* combobox clicked */
	    if (title)   /* 'goto page' mode: cannot change category */
	        continue;
	    if ( LibIconClick( &FLIcons[FLCOMBO], &tsts ) == TRUE )  {
		int ncat;
		ncat = selectcategory(cat,NULL); /* display category menu */
		if (ncat != 0xff
#ifndef TEST1
		    && ncat != 0 /* 'All' not yet implemented */
#endif		    
		    && ncat != cat) {
		    cat=ncat;
		    getcategoryname(cat, name);
		    LibGdsClr(2,14, 124,23);
		    LibPutProStr(IB_PFONT1, 3,15, name, 124-3); /* redraw label */
		    LibPutDisp();
		    
		    populatelist(cat,title,0); /* load items from new category */
		    if (FLScb.vol >0) 
		        selBarReverse();/* show selection */
		    LibScrollPrint(FLScb); /* reinitialize the scrollbar */
		    if (FLScb.vol<=FLScb.dsp) 
		        LibScrollArrowPrint(FLScb, 0);
		    LibScrollArrowPrint(FLScb, FLScb.vol>1 ? SCR_UP_MASK : SCR_ALL_MASK);
                    SYS_MEMO_CATEGORY = cat;
		}
	    }
	    break;

        case OBJ_LPSW_UP:  /*ceV3.1 added Action-Wheel support*/
        case OBJ_LPSW_DOWN:
	case OBJ_FLSCB:
	    selBarReverse(); /* hide selection */
            if ( tsts.obj == OBJ_LPSW_UP)  /*ceV3.1 added Action-Wheel support*/
                scrlreq = 0;
            else if ( tsts.obj == OBJ_LPSW_DOWN)
                scrlreq = 1;
            else
  	        scrlreq = LibScrollClick(&tsts, &FLScb);
	    switch (scrlreq) {
	    case -1:
		/* mustRedraw = FALSE; */
		break;
	    case 0: /* scroll up */
		if (sel>0) 
		    sel--;
		else if (FLScb.pos>0) 
		    { FLScb.pos--; sel--; }
		break;
	    case 1: /* scroll down */
		if (sel < FLScb.vol - FLScb.pos -1) {
		    if (sel >= DISPITEMS-1) 
		        FLScb.pos++;
		    sel++;
		}
		break;
	    case 2:
	    case 4:
		/* noting to do, scrollbar position was updated by
		   LibScrollClick */
		break;
	    }
	    if (FLScb.pos != oldpos) 
		populatelist(cat,title,FLScb.pos-oldpos);
	    LibScrollPrint(FLScb);
	    if (FLScb.vol<=FLScb.dsp)
		LibScrollArrowPrint(FLScb, 0);
	    LibScrollArrowPrint(FLScb,
				((sel>0 || FLScb.pos>0)? 0 : SCR_UP_MASK) |
				((sel<FLScb.vol-FLScb.pos-1)? 0 : SCR_DWN_MASK));
	    selBarReverse(); /* show selection */
	    break;
	case OBJ_FLLIST:
	    { 
		int nsel = (tsts.y - 28) / 10; /* this item was clicked */
		if (nsel >= FLScb.vol - FLScb.pos) /* clicked beyond last item */
		    continue; 

		selBarReverse(); /* hide selection */
		if (sel == nsel) { /* item clicked for the second time - return it */
                    retval = TRUE;
		    continue; /* in fact: break the loop */
		}
		sel = nsel;
		LibScrollArrowPrint(FLScb, 0); /* redraw scrollbar arrows */
		LibScrollArrowPrint(FLScb,
				    ((sel>0 || FLScb.pos>0)? 0 : SCR_UP_MASK) |
				    ((sel<FLScb.vol-FLScb.pos-1)? 0 : SCR_DWN_MASK));
		selBarReverse(); /* show selection */
	    }
	    break;
	case OBJ_HIC_ESC:
	    /* <esc> pressed - return no value. */
	    retval = FALSE;
	    continue; /* in fact: break the loop */
        
        case OBJ_LPSW_PUSH: /*ceV3.1 Action-Wheel support */
 	    if (FLScb.vol == 0) 
 	        continue;   /* handle empty file lists */
	    selBarReverse(); /* hide selection */
	    retval = TRUE;
	    continue; /* in fact: break the loop */
#ifdef TEST
        case OBJ_FLPREVIEWBUTTON:
            if ( LibIconClick( &FLIcons[FLPREVIEWBUTTON], &tsts ) == TRUE ) {
                char *filetxt;
                int i=0,li=0,lc=0;
                filetxt = getNextPrevPageContentsAndCat(NULL, SEARCH_THIS_PAGE, &fps[sel], &cats[sel]);
                previewline[0]=previewbuf;
                while (*filetxt!=0 && i<MAXPREVIEW && lc<MAXPREVIEWLINES) {
                    if ( li>=MAXPREVIEWWIDTH /*max chars per paragraph reached (=per line) */
                         || *filetxt==0xfe || *filetxt=='\r') {
                        /*don't cut lines is better
                          if (*filetxt!=0xfe && *filetxt!='\r') /*show that lines is cut *
                            previewbuf[i++]=0xD0;
                        while (*filetxt!=0xfe && *filetxt!='\r' && *filetxt!=0) /* goto end of line *
                            filetxt++;
                        */
                        while (*filetxt==0xfe || *filetxt=='\r') /* skip empty lines */
                            filetxt++;
                        li=0;
                        previewbuf[i]=0;
                        lc++;
                        previewline[lc]=previewbuf+i+1;
                    } else {
                        previewbuf[i] = *filetxt;
                        if (*filetxt!=0)
                            filetxt++;
                        li++;
                    }
                    i++;
                };
                /*don't cut lines is better
                if (*filetxt!=0xfe && *filetxt!='\r' && *filetxt!=0) /*show that lines is cut *
                    previewbuf[i++]=0xD0;*/
                previewbuf[i]=0;
                if (li>0)
                    lc++;
                previewline[lc]=previewbuf+i; /* last line points to (same) 0 */
                /*LibPutMsgDlg("%s", previewbuf );*/
                LibOpenWindow( 2,13, 155,147 );
	        LibGdsBox(2,13, 156,159); /* Box leaves one blank pixel column at the right */
                printCharArray( 5, 15, previewline, 158, IB_CG57FONT );
                waitForClick();
                LibCloseWindow();
            }
            break;
#endif
	}
	LibPutDisp();
	/* LibPutMsgDlg("v: %d p: %d\rs: %d\rarr: %d", 
	   FLScb.vol, FLScb.pos, sel,
	   ((sel>0 || FLScb.pos>0)? 0 : SCR_UP_MASK) |
	   ((sel<FLScb.vol-FLScb.pos-1)? 0 : SCR_DWN_MASK)); */
    }
    
    if (retval == TRUE)
	if (loadpage(txtbuf, NULL, SEARCH_THIS_PAGE, &fps[sel], &cats[sel]) ) {
	    *p_fp = fps[sel];
	    *p_cat = cats[sel];
	} else
	    retval = FALSE;
    LibTchRelease(); /* destroy local touch table */
    LibCloseWindow();
    LibPutDisp(); /* is this really needed? */
    return retval;
}

/* end of new stuff :MW */

/* load next/prev page after *p_fp with a certain title (or title==null to load any memo)
   (search for first page, if *p_pf==0xffff; last page==0xfffe */
/* txtbuf and title are allowed to point to the same buffer -> title will be overridden by text */
/* *p_fp is set to new page */
/* returns TRUE for success, FALSE otherwise */
int loadpage(char *txtbuf, char *title, t_searchdir d, word *p_fp, int *p_cat)
{
    char *res;
    /*LibPutMsgDlg("t=%s\rd=%i fp=%x c=%i", title, d, *p_fp, *p_cat );*/
    res = getNextPrevPageContentsAndCat( title, d, p_fp, p_cat);
    if (res==NULL)
        return FALSE;
    strcpy(txtbuf, res);
    return TRUE;
}


/*ce internal load function */
/* load next/prev page (according to given 'dir'ection) after a given memo
   at '*p_fp' with a certain title (or title==null to load any memo)
   (search for first page, if *p_pf==0xffff; last page==0xfffe)
   in the category 'cat' */
/* on success, '*p_fp' is changed to the new file-pointer */
/* returns NULL if no page/memo was found, otherwise a pointer to the 
   memo-contents */
/* and if (d!=SEARCH_THIS_PAGE) also sets *p_cat to the one of the file, but leaves it untouched otherwise */
static char* getNextPrevPageContentsAndCat(char *title, t_searchdir d, word *p_fp, int *p_cat)
{
    int found;
    int fileok;
    int l;
    FILE_BUF fb;
    FILE_INFO finf;

    /*LibPutMsgDlg("t=%s\rd=%i fp=%x c=%i", title, d, *p_fp, *p_cat );*/

    l = (title ? strlen(title) : 0);
    fb.fsb_main_entry_ = CURRENT_FILEREAD_MAIN_ENTR(*p_cat);	/* main mode code */
    fb.fsb_sub_entry_  = *p_cat;	/* Sub mode code (memo category) */
    fb.fsb_scrt_info_  = CURRENT_FILE_MODE;
    finf.fp      = *p_fp;
    finf.kind    = FILE_KIND_TEXT; /* Memos are saved as text */

    /*LibPutMsgDlg("m=%i sm=%i fm=%x fp=%x", fb.fsb_main_entry_, fb.fsb_sub_entry_, fb.fsb_scrt_info_ , finf.fp );*/

    /* if we search for first or last page we ignore value of parameter d */
    if ( finf.fp == 0xffff)
        d = SEARCH_NEXT_PAGE;
    else if ( finf.fp == 0xfffe )
        d = SEARCH_PREV_PAGE;
    
    do { /* iterate over files until right title is found */
        if ( d == SEARCH_NEXT_PAGE || finf.fp == 0xfffe ) /* search forward or for very last item */
            fileok = LibFileFindNext(&fb,&finf,0x00);
        else if ( d == SEARCH_PREV_PAGE )
            fileok = LibFileFindPrev(&fb,&finf,0x00);
        if (fileok || d == SEARCH_THIS_PAGE )
            fileok = LibFileRead(&fb,&finf);
        if (fileok) {
            if ( l==0 )
                found=TRUE;
            else {
                found = !strncmp(title, fb.fbuf.text.fsb_text_buf_, min(l,MAX_TITLE_LEN) ); /* check title prefix */
                if (found && l<MAX_TITLE_LEN ) { /* check whether loaded title has same length */
                                                 /* (-> titles are equal, test whether searchtitle is not just a prefix of the loaded one - only for titles longer than the maximum, this is not checked ) */
                    char *tend = fb.fbuf.text.fsb_text_buf_+l;
                    while ( *tend==' ' ) /*skip spaces*/
                      tend++;
                    found = IS_TITLE_ENDCHAR(*tend);
                }
                if (d == SEARCH_THIS_PAGE && !found)
                    return NULL;
            }
        } 
    } while ( fileok && !found );
    
    if (!fileok || !found)
        return NULL;
    
    *p_fp = finf.fp;
    if (d!=SEARCH_THIS_PAGE)
        *p_cat = fb.fsb_sub_entry_;  /* set real category of read file, does not work for SEARCH_THIS_PAGE */
    /*LibPutMsgDlg("loaded file %x\rwith c=%i", finf.fp, *p_cat );*/
    return fb.fbuf.text.fsb_text_buf_;
}

/* save text in *textbuf in the textfile at *p_fp in categorie *p_cat */
/* fp==NEWFILE -> ask user for categorie and save as a new memo */
/* *p_fp is changed after saving !! */
/* returns success (TRUE/FALSE) or HALF if user canceled category selection */
int savetext(char *txtbuf, word *p_fp, int *p_cat) {
    return savetextAt( txtbuf, p_fp, p_cat, NEWFILE );
}

/* same as above, but sorts file in at a given place */
int savetextAt(char *txtbuf, word *p_fp, int *p_cat, word prev_fp ) {
  FILE_BUF fb;
  FILE_INFO finf;
  
  if ( *p_cat==-1 ) {  /* category not selected, yet */
      int cat = selectcategory(*p_cat,"Save to ...");
      if ( cat == 0xff) 
          return HALF; 
      *p_cat = cat;
  }

  LibWinIcnMsg(ICON_COFFEE,381,0); /* Please_wait... */
  fb.fsb_main_entry_ = CURRENT_FILEWRITE_MAIN_ENTR(*p_cat) ;
  fb.fsb_sub_entry_  = *p_cat;	/* Sub mode code (memo category) */
  fb.fsb_scrt_info_  = CURRENT_FILE_MODE;	
  finf.fp      = *p_fp;
  finf.kind    = FILE_KIND_TEXT; /* Memos are saved as text */
  strcpy(fb.fbuf.text.fsb_text_buf_, txtbuf);
  /*LibPutMsgDlg("Will write to\rc=%d fp=%i", *p_cat, finf.fp);*/
  if ( LibFileWrite(&fb,&finf)==FALSE ) {
      LibCloseWindow(); /* LibWinIcnMsg's window */
      return FALSE;
  }
  *p_fp = finf.fp;
  
  if (prev_fp!=NEWFILE) {
      LibFileExch( &fb, &finf, prev_fp);
  }
  LibCloseWindow(); /* LibWinIcnMsg's window */
  return TRUE;
}


int deletefile(word fp) {
    FILE_BUF unusedfb;
    FILE_INFO finf;
    finf.fp      = fp;
    finf.kind    = FILE_KIND_TEXT; 
    return LibFileRemove( &unusedfb, &finf );
}
